#!/bin/ksh
# **********************************************************************
# * Licensed Materials - Property of IBM
# * 5724-C04
# * Copyright IBM Corporation 2021 All Rights Reserved
# **********************************************************************
#
# This script will display missing proccess information from a RunInfo
# file, or add the missing process information to a RunInfo file.
# This script can process one ITM installation or locate and process all
# ITM installations using the ITM boot scripts and/or ITM service files.

Use_report() {
    print -u2 "\n$PROGNAME [-a] [-d] [-h install_dir] [-q] [-s seconds]";
    print -u2 "\t-a\t\t Force processing of all ITM installations.";
    print -u2 "\t-d\t\t Display only, do not modify RunInfo and RunInfo_Description.";
    print -u2 "\t-h install_dir\t Specify a single ITM installation to process.";
    print -u2 "\t-q\t\t Do not display \"Found\" entries.";
    print -u2 "\t-s seconds\t Specify the number of seconds to wait before processing.";
    print -u2 "\tDefault with no options is to process all ITM installations";
    print -u2 "\t\t\t and modify RunInfo/RunInfo_Description.";
    print -u2 "\tIf CANDLEHOME is set and exported, it is as if -h is specified.";
    print -u2 "\tIf both -a and -h are specified, -a overrides -h.\n";
    exit 2;
}

resolveDir() {
  #---------------------------------------------------------------------#IV67523
  # Perform directory canonicalization. In other words, the input       #IV67523
  # directory string is converted to a fully qualified name (no         #IV67523
  # wildcards, "..", ".", and so on. For example, "/tmp/.." resolves    #IV67523
  # to "/". The input directory string must refer to a directory that   #IV67523
  # exists.                                                             #IV67523
  #                                                                     #IV67523
  # AIX and Linux provide the "readlink" command to perform this        #IV67523
  # function. If readlink does not exist, we use "cd -P". This          #IV67523
  # approach changes the directory *and* sets PWD to be the             #IV67523
  # canonicalized directory name. So, we switch to the directory, grab  #IV67523
  # its name, and switch back to the original working directory.        #IV67523
  #                                                                     #IV67523
  # Solaris has some sort of buffering issue when the cd shell command  #IV67523
  # sets $PWD and when the pwd shell command returns the value. In      #IV67523
  # in some cases, one or both of these leave extra garbage on the end  #IV67523
  # of the output value. And of course, this creates problems for us    #IV67523
  # later when we try to use it. We get around the problem by using     #IV67523
  # the /usr/bin/pwd binary, which always returns the correct value     #IV67523
  # without corruption.                                                 #IV67523
  #---------------------------------------------------------------------#IV67523
  typeset result;                           # localize output variable  #IV67523
  result=$(readlink -f ${1} 2> /dev/null);  # try readlink              #IV67523
  if [[ ${?} -ne 0 ]]; then                 # if no readlink or failed  #IV67523
    cd -P ${1} > /dev/null 2>&1;            # use cd to change/resolve  #IV67523
    if [[ ${?} -ne 0 ]]; then               # if cd failed...           #IV67523
      result="";                            # indicate same to caller   #IV67523
    else                                    # get result from cd        #IV67523
      if [[ "${thisOS}" = "SunOS" ]]; then  # $PWD/pwd shell broken...  #IV67523
        result=$(/usr/bin/pwd);             # ...use pwd binary         #IV67523
      else                                  # $PWD is ok...             #IV67523
        result="${PWD}";                    # ...use its value          #IV67523
      fi;                                   # (elif($os="sol"))         #IV67523
      cd ${OLDPWD} > /dev/null 2>&1;        # restore working directory #IV67523
    fi;                                     # (else($?!=0))             #IV67523
  fi;                                       # (if($?!=0))               #IV67523
  print "${result}";                        # pass result to caller     #IV67523
}                                           # (resolveDir())            #IV67523

# Method to check if a specified string is used within
# the RunInfo or RunInfo_Description file
check_for_string(){
   filename=$1;         # Path of RunInfo or RunInfo_Description file
   thisPID=$2;
   target="$3";         # String to search for
   found=$(grep "$target" "$filename" 2> /dev/null); # Search file string
   # If found, print message, else print messsage and add string to file.
   if [[ -n "$found" ]]; then
      if [[ "$QUIET" == false ]]; then
         print "Found $filename record \"$target\"" 
      fi
   else
      print "Missing $filename record \"$target\""
      if [[ "$REPAIR" == "true" ]]; then
         print "Adding $filename record \"$target\"" 
         print $CR$target >> $filename
      fi
   fi
}

###################################################################
###################################################################
#                         Main Routine                            #
###################################################################
###################################################################
unalias -a;
PROGNAME=$(basename $0)

CR="
";

OS=$(uname);
if [[ "${OS}" = "SunOS" ]]; then
   tailCMD="tail";    psFopt="-o ruser,pid,ppid,c,stime,tty,time,args";
elif [[ "${OS}" = "Linux" ]]; then
   tailCMD="tail -n"; psFopt="-o ruser,pid,ppid,c,stime,tty,time,args";
else
  tailCMD="tail -n"; psFopt="-f";
fi

DISPLAY=false;
REPAIR=true;
QUIET=false;
ONEHOME="";
SLEEP="";

#Determines a mode to run the script in, either display (-d) or repair (no flag)
while getopts ":adqs:h:" OPTS ; do
   case $OPTS in
      a) ONEHOME=false
         ;;
      d) DISPLAY=true
         REPAIR=false
         ;;
      h) CANDLEHOME=$OPTARG
         ;;
      q) QUIET=true
         ;;
      s) SLEEP=$OPTARG
         ;;
      ?) Use_report
         ;;
      *) Use_report
         ;;
   esac
done

[[ -n "$SLEEP" && "$SLEEP" == @(\d*) ]] && sleep $SLEEP

[[ "$DISPLAY" == "true" ]] && print "Display only option enabled"
[[ "$REPAIR" == "true" ]] && print "Repair option enabled"

if [[ -z "$ONEHOME" ]]; then
   if [[ -z "$CANDLEHOME" ]]; then
      ONEHOME=false;
   else
      if [[ ! -d "$CANDLEHOME" || $(print "$CANDLEHOME" | wc -w) -ne 1 ]]; then
         ONEHOME=false;
      else
         ONEHOME=true;
      fi
   fi
fi

if [[ "$ONEHOME" == "true" ]]; then
   homes="$CANDLEHOME";
else
   # Find boot files
   initd=$(ls -d /etc/init.d/ITMAgents* 2> /dev/null);
   systemd=$(systemctl list-unit-files ITMAgents\* 2> /dev/null | grep service | cut -d' ' -f1);
   rcitm=$(ls -d /etc/rc.itm* 2> /dev/null);
   # Find ITM installations
   homes="";
   if [[ -n "$initd" ]]; then
      for init in $initd; do
         homes="${homes}${CR}$(grep CANDLEHOME= $init 2> /dev/null | cut -d= -f2 | tr -d \")";
      done
   fi
   if [[ -n "$systemd" ]]; then
      for sysd in $systemd; do
         homes="${homes}${CR}$(systemctl cat $sysd | grep CANDLEHOME= 2> /dev/null | cut -d= -f3)";
      done;
   fi
   if [[ -n "$rcitm" ]]; then
      for rc in $rcitm; do
         homes="${homes}${CR}$(grep CANDLEHOME= $rc 2> /dev/null | cut -d= -f2 | tr -d \")";
      done;
   fi
fi

# Genrate RunInfo and RunInfo_Description records for each CANDLEHOME
if [[ -n "$homes" ]]; then
   homes=$(print -- "$homes" | sort -u); RIlist=""; RIDlist="";
   for CH in $homes; do
      [[ -f $CH/bin/ITMhostName ]] || continue;
      CH2=$(resolveDir "$CH");
      fRI=$CH/config/.ConfigData/RunInfo
      fRID=$CH/config/.ConfigData/RunInfo_Description
      [[ -f $CH/bin/ITMLangC ]] && ITMps="$CH/bin/ITMLangC ps" || ITMps="ps";
      hosts=$(print -- $($CH/bin/ITMhostName -l -a)); host=${hosts%% *};
      procs=$($ITMps -e $psFopt 2> /dev/null | egrep "$CH/|$CH2/" | grep -v grep);
      procs=$(print -- "$procs" | egrep -ve 'ITMLangC|regenRuninfo|ITMinstall.JFrame1|CandleManage|CandleAgent|CandleServer|itmcmd' | grep -v grep);
      procs=$(print -- "$procs" | egrep -ve 'kdsmain|cq.sh|stopcnps.sh|/iw/|/iu/' | grep -v grep);
      procs=$(print -- "$procs" | egrep -ve 'kcawd|daemon|ifstat|kux_vmstat|nfs_stat|mount_stat|kuxdstat' | grep -v grep);
      procs=$(print -- "$procs" | egrep -ve 'dchome|java ' | grep -v grep);
      pids=$(print -- "$procs" | awk '{print $2}');

      if [[ -n "$pids" ]]; then
         RIlist=""; RIDlist="";
         for pid in $pids; do
            proc=$($ITMps -p $pid $psFopt 2> /dev/null | $tailCMD -1);
            startIndex=$(print -- $proc | awk '{if (($5 ~ /[A-Z][a-z][a-z]_?[0-9]/) || ($5 ~ /[0-9][0-9]:/) || ($5 ~ /[0-9][0-9][0-9][0-9]/)) print 8; else print 9}');
            psinfo=$(print -- $proc | cut -d" " -f$startIndex- );
            trimmed=$(print -- $psinfo | sed "s@/bin/ksh @@" | sed "s@$CH@@" | sed "s@$CH2@@");
            plat=$(print -- $trimmed | cut -d/ -f2 );
            pp=$(print -- $trimmed | cut -d/ -f3 );
            inst=$(print -- $trimmed | cut -d" " -f2);
            [[ $(print -n -- $pp | wc -c) != 2 ]] && continue

            # Figure out the Instance information for running agent instances
            [[ -z "$inst" || "$inst" == "-c" || "$inst" == @(/$plat/$pp/*) ]] && inst=None;
            [[ "$inst" == @(*\.cfg) ]] && inst=$(print -- $inst | sed "s@$CH/config/\(.*\).cfg@\1@" | sed "s@$CH2/config/\(.*\).cfg@\1@");
            [[ "$pp" == hd || "$pp" == pa || "$pp" == sy || "$pp" == th ]] && inst=None;
            [[ "$pp" == ms ]] && inst=$(grep CELLNAME $CH/config/.ConfigData/kmsenv | grep -v '^default' | cut -d'|' -f1);
            [[ "$pp" != ud ]] && inst=$(print -- $inst | sed "s@${host}_@@");
            [[ "$pp" == mq ]] && inst=$(print -- $inst | sed "s@mq_@@");

            RI=$(print -- "$host|$pp|$pid|$inst|$plat|");
            RID=$(print -- "${pid}.description=$psinfo");

            check_for_string "$fRI" "$pid" "$RI"
            check_for_string "$fRID" "$pid" "$RID"

         done
      fi
   done
fi
